﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.IO;
using System.Drawing;
using System.Net;
using System.Web.Hosting;
using EPiServer.Core;
using EPiServer;
using EPiServer.Web.Hosting;
using EPiServer.Web.WebControls;
using System.Drawing.Imaging;
using EPiServer.Data.Dynamic;
using BearWare.PixlrGadget.Models;

namespace BearWare.PixlrGadget.Content
{
    public partial class GIR : SimplePage
    {
        private string usedPath { get; set; }

        protected void Page_Load(object sender, EventArgs e)
        {
            Guid gadgetId = new Guid(Request.QueryString["gadgetid"]);
            bool forceNewFile = bool.Parse(Request.QueryString["forcenew"]??"false");

            string orgTitle = Request.QueryString["orgTitle"];
            if (!string.IsNullOrEmpty(orgTitle))
                orgTitle = orgTitle.Substring(0, orgTitle.LastIndexOf('.'));
            bool titleChanged = orgTitle != Request.QueryString["title"];

            bool versioningSupport = Request.QueryString["versioningsupport"] == "true";
            
            Stream imageStream = new WebClient().OpenRead(Request.QueryString["image"]);
            Bitmap image = (Bitmap)System.Drawing.Image.FromStream(imageStream);

            UnifiedDirectory ud = null;
            #region File loaded from user's computer
            if (Request.QueryString["state"] == "new") // image not loaded from site
            {
                if (Request.QueryString["targetfolder"] != null)
                    ud = HostingEnvironment.VirtualPathProvider.GetDirectory(Request.QueryString["targetfolder"]) as UnifiedDirectory;
                // if no target folder is specified, try using default folder
                if (ud == null)
                {
                    ud = HostingEnvironment.VirtualPathProvider.GetDirectory(DefaultFolder) as UnifiedDirectory;

                    // in case of missconfiguration of default folder - save in page folder for start page
                    if (ud == null)
                    {
                        PageData startPage = DataFactory.Instance.GetPage(PageReference.StartPage);
                        ud = startPage.GetPageDirectory(true);
                    }
                }
                SaveAndAvoidDuplicateFilenames(ud.VirtualPath, Request.QueryString["title"], Request.QueryString["type"], image);
            }
            #endregion
            #region File loaded from site
            else
            {
                bool targetFolderSet = false;
                if (Request.QueryString["targetfolder"] != null)
                {
                    ud = HostingEnvironment.VirtualPathProvider.GetDirectory(Request.QueryString["targetfolder"]) as UnifiedDirectory;
                    targetFolderSet = true;
                }
                // if no target folder i specified, try using the original file path
                if (ud == null)
                {
                    ud = HostingEnvironment.VirtualPathProvider.GetDirectory(Request.QueryString["folderpath"]) as UnifiedDirectory;
                    // in case of error - save in page folder for start page
                    if (ud == null)
                    {
                        PageData startPage = DataFactory.Instance.GetPage(PageReference.StartPage);
                        ud = startPage.GetPageDirectory(true);
                    }
                }
                // check if we should create a new version of a file
                if (versioningSupport && !forceNewFile && !targetFolderSet && !titleChanged)
                {
                    VersioningDirectory vd = ud as VersioningDirectory;
                    string vfpath = ((VirtualPathVersioningProvider)vd.Provider).ResolveFile(new Guid(Request.QueryString["guid"]));
                    VersioningFile vf = HostingEnvironment.VirtualPathProvider.GetFile(vfpath) as VersioningFile;
                    usedPath = vfpath;
                    vf.BypassAccessCheck = true;
                    vf.CheckOut();

                    using (Stream writeStream = vf.Open(FileMode.OpenOrCreate, FileAccess.ReadWrite))
                    {
                        image.Save(writeStream, Encoder(vf.Extension));
                    }

                    vf.CheckIn(Translate("/BearWare/PixlrPlugin/VersioningFileComment"));
                }
                else
                {
                    if(targetFolderSet)
                        SaveAndAvoidDuplicateFilenames(Request.QueryString["targetfolder"], Request.QueryString["title"], Request.QueryString["type"], image);
                    else
                        SaveAndAvoidDuplicateFilenames(Request.QueryString["folderpath"], Request.QueryString["title"], Request.QueryString["type"], image);
                }
            }
            #endregion

            ViewModel vm = BearWare.PixlrGadget.Data.LoadViewModel(gadgetId);
            BearWare.PixlrGadget.Models.FileInfo fi = vm.FileInfo;
            fi.SavedFilePath = usedPath;
            BearWare.PixlrGadget.Data.SaveFileInfo(gadgetId, fi);
        }

        private void SaveAndAvoidDuplicateFilenames(string folder, string title, string extension, Bitmap image)
        {
            bool duplicate = false;
            int i = 0;
            string path = folder + title + "." + extension;
            string orgPath = path;
            while (CheckIfFileExists(path))
            {
                path = folder + title + i.ToString() + "." + extension;
                duplicate = true;
                i++;
            }

            if (duplicate)
            {
                SaveImage(path, image);
                usedPath = path;
            }
            else
            {
                SaveImage(orgPath, image);
                usedPath = orgPath;
            }
        }

        private bool CheckIfFileExists(string filepath)
        {
            return HostingEnvironment.VirtualPathProvider.FileExists(filepath);
            //UnifiedFile file = HostingEnvironment.VirtualPathProvider.GetFile(filepath) as UnifiedFile;
            //if (file == null)
            //    return false;
            //return true;
        }

        public void SaveImage(string path, Bitmap img)
        {
            UnifiedFile file = GetFile(path);
            using (Stream stream = file.Open(FileMode.OpenOrCreate, FileAccess.Write))
            {
                img.Save(stream, Encoder(path));
            }
        }

        private ImageFormat Encoder(string extension)
        {
            extension = extension.ToLower();

            if (extension.EndsWith("jpg") || extension.EndsWith("jpeg"))
                return ImageFormat.Jpeg;
            if (extension.EndsWith("bmp"))
                return ImageFormat.Bmp;
            return ImageFormat.Png;
        }

        private UnifiedFile GetFile(string virtualPath)
        {
            UnifiedFile file = HostingEnvironment.VirtualPathProvider.GetFile(virtualPath) as UnifiedFile;
            if (file == null)
            {
                string virtualDir = VirtualPathUtility.GetDirectory(virtualPath);
                UnifiedDirectory directory = HostingEnvironment.VirtualPathProvider.GetDirectory(virtualDir) as UnifiedDirectory;
                return directory.CreateFile(VirtualPathUtility.GetFileName(virtualPath));
            }
            return file;
        }

        #region Accessors
        public string DefaultFolder
        {
            get
            {
                //PixlrPlugin setting = new PixlrPlugin();
                //PlugInSettings.AutoPopulate(setting);
                //return setting.DefaultFolder;
                return "/Global/";
            }
        }
        #endregion
    }
}