﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using BearWare.PixlrGadget.Models;
using EPiServer.Data.Dynamic;
using EPiServer.Core;
using EPiServer;

namespace BearWare.PixlrGadget
{
    public static class Data
    {
        static private DynamicDataStore<FileInfo> _fileInfoStore;
        static private DynamicDataStore<FileInfo> FileInfoStore
        {
            get
            {
                if (_fileInfoStore == null)
                {
                    _fileInfoStore = DynamicDataStore<FileInfo>.CreateStore(false);
                }
                return _fileInfoStore;
            }
        }

        static private DynamicDataStore<Settings> _settingsStore;
        static private DynamicDataStore<Settings> SettingsStore
        {
            get
            {
                if (_settingsStore == null)
                {
                    _settingsStore = DynamicDataStore<Settings>.CreateStore(false);
                }
                return _settingsStore;
            }
        }

        static private Settings LoadSettings(Guid gadgetId)
        {
            Settings data = SettingsStore.Find("GadgetId", gadgetId).FirstOrDefault();
            if (data == null)
            {
                data = new Settings(gadgetId);
                data.EditorHeight = 800;
                data.PreviewSize = 150;
            }

            return data;
        }
        static private FileInfo LoadFileInfo(Guid gadgetId)
        {
            FileInfo data = FileInfoStore.Find("GadgetId", gadgetId).FirstOrDefault();
            if (data == null)
            {
                data = new FileInfo(gadgetId);
                data.FilePath = string.Empty;
                data.SaveAsNew = false;
                data.SavedFilePath = string.Empty;
                data.TargetFolder = string.Empty;
            }

            return data;
        }
        static public ViewModel LoadViewModel(Guid gadgetId)
        {
            ViewModel vm = new ViewModel(LoadFileInfo(gadgetId), LoadSettings(gadgetId));

            PageData start = DataFactory.Instance.GetPage(PageReference.StartPage);
            vm.StartPageId = PageReference.StartPage.ID;
            vm.StartPageParentId = start.ParentLink.ID;
            vm.StartPageFolderId = (start["PageFolderId"] as int?) ?? 0;

            return vm;
        }

        static public void SaveSettings(Guid gadgetId, Settings settings)
        {
            Settings saveData = LoadSettings(gadgetId);
            saveData.EditorHeight = settings.EditorHeight;
            saveData.PreviewSize = settings.PreviewSize;
            SettingsStore.Save(saveData);
        }

        static public void SaveFileInfo(Guid gadgetId, FileInfo fileinfo)
        {
            FileInfo saveData = LoadFileInfo(gadgetId);
            saveData.FilePath = fileinfo.FilePath;
            saveData.SaveAsNew = fileinfo.SaveAsNew;
            saveData.SavedFilePath = fileinfo.SavedFilePath;
            saveData.TargetFolder = fileinfo.TargetFolder;
            FileInfoStore.Save(saveData);
        }

        static public void ClearGadgetValues(Guid gadgetId)
        {
            foreach (Settings setting in SettingsStore.Find("gadgetId", gadgetId))
            {
                SettingsStore.Delete(setting);
            }

            foreach (FileInfo fileinfo in FileInfoStore.Find("gadgetId", gadgetId))
            {
                FileInfoStore.Delete(fileinfo);
            }
        }
    }
}
